@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2009  Tom Cato Amundsen
+# Copyright (C) 2009, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,12 +22,8 @@ import logging
 import os
 import StringIO
 import subprocess
-import sys
-import time
-import weakref
 
-import gobject
-import gtk
+from gi.repository import Gtk
 
 from solfege.esel import SearchView
 
@@ -43,34 +39,33 @@ from solfege import filesystem
 from solfege import gu
 from solfege import frontpage as pd
 from solfege import lessonfile
-from solfege import lessonfilegui
 from solfege import osutils
 
-class LessonFilePreviewWidget(gtk.VBox):
+class LessonFilePreviewWidget(Gtk.VBox):
     def __init__(self, model):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         self.m_model = model
         self.set_size_request(200, 200)
-        l = gtk.Label()
+        l = Gtk.Label()
         l.set_alignment(0.0, 0.5)
         l.set_markup("<b>Title:</b>")
-        self.pack_start(l, False)
-        self.g_title = gtk.Label()
+        self.pack_start(l, False, False, 0)
+        self.g_title = Gtk.Label()
         self.g_title.set_alignment(0.0, 0.5)
-        self.pack_start(self.g_title, False)
-        l = gtk.Label()
+        self.pack_start(self.g_title, False, False, 0)
+        l = Gtk.Label()
         l.set_alignment(0.0, 0.5)
         l.set_markup("<b>Module:</b>")
-        self.pack_start(l, False)
-        self.g_module = gtk.Label()
+        self.pack_start(l, False, False, 0)
+        self.g_module = Gtk.Label()
         self.g_module.set_alignment(0.0, 0.5)
-        self.pack_start(self.g_module, False)
-        l = gtk.Label()
+        self.pack_start(self.g_module, False, False, 0)
+        l = Gtk.Label()
         l.set_alignment(0.0, 0.5)
         l.set_markup("<b>Used in topcis:</b>")
-        self.pack_start(l, False)
-        self.g_topic_box = gtk.VBox()
-        self.pack_start(self.g_topic_box, False)
+        self.pack_start(l, False, False, 0)
+        self.g_topic_box = Gtk.VBox()
+        self.pack_start(self.g_topic_box, False, False, 0)
         self.show_all()
     def update(self, dlg):
         fn = dlg.get_preview_filename()
@@ -85,13 +80,13 @@ class LessonFilePreviewWidget(gtk.VBox):
                 self.g_module.set_text(lessonfile.infocache.get(fn, 'module'))
                 self.g_ok_button.set_sensitive(True)
                 for x in self.m_model.iterate_topics_for_file(fn):
-                    l = gtk.Label(x)
+                    l = Gtk.Label(label=x)
                     l.set_alignment(0.0, 0.5)
-                    self.g_topic_box.pack_start(l, False)
+                    self.g_topic_box.pack_start(l, False, False, 0)
                 if not self.g_topic_box.get_children():
-                    l = gtk.Label(u"-")
+                    l = Gtk.Label(label=u"-")
                     l.set_alignment(0.0, 0.5)
-                    self.g_topic_box.pack_start(l, False)
+                    self.g_topic_box.pack_start(l, False, False, 0)
             except (lessonfile.InfoCache.FileNotFound,
                     lessonfile.InfoCache.FileNotLessonfile), e:
                 self.g_title.set_text(u'')
@@ -101,31 +96,31 @@ class LessonFilePreviewWidget(gtk.VBox):
         self.show_all()
         return True
 
-class SelectLessonFileDialog(gtk.FileChooserDialog):
+class SelectLessonFileDialog(Gtk.FileChooserDialog):
     def __init__(self, parent):
-        gtk.FileChooserDialog.__init__(self, _("Select lesson file"),
+        Gtk.FileChooserDialog.__init__(self, _("Select lesson file"),
             parent=parent,
-            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
+            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,))
         self.set_select_multiple(True)
         pv = LessonFilePreviewWidget(parent.m_model)
-        pv.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
+        pv.g_ok_button = self.add_button("gtk-ok", Gtk.ResponseType.OK)
         pv.g_ok_button.set_sensitive(False)
         pv.show()
         self.set_preview_widget(pv)
         self.connect('selection-changed', pv.update)
 
 
-class SelectLessonfileBySearchDialog(gtk.Dialog):
+class SelectLessonfileBySearchDialog(Gtk.Dialog):
     def __init__(self):
-        gtk.Dialog.__init__(self, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT))
+        Gtk.Dialog.__init__(self, buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.ACCEPT))
         view = SearchView(_('Search for exercises. Each exercise you click will be added to the section of the front page.'),
             fields=['link-with-filename-tooltip', 'module'])
         view.on_link_clicked = self.on_link_clicked
-        self.vbox.pack_start(view)
+        self.vbox.pack_start(view, True, True, 0)
         self.show_all()
     def on_link_clicked(self, widget, filename):
         self.m_filename = filename
-        self.response(gtk.RESPONSE_OK)
+        self.response(Gtk.ResponseType.OK)
 
 
 def editor_of(obj):
@@ -143,113 +138,116 @@ def parent_page(obj):
     """
     p = obj
     while True:
-        p = p.m_parent
+        try:
+            p = p.m_parent
+        except AttributeError:
+            return None
         if isinstance(p, Page):
             return p
         if p is None:
             return None
 
-class Section(gtk.VBox):
+class Section(Gtk.VBox):
     """
     A section consists of a heading and a list of links.
     self.g_link_box is a vbox that contains the links.
     """
     def __init__(self, model, parent):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         self.m_model = model
         self.m_parent = parent
         assert isinstance(model, pd.LinkList)
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.set_spacing(6)
-        self.pack_start(hbox, False)
+        self.pack_start(hbox, False, False, 0)
         # This is displayed and used when we edit the heading
-        self.g_heading_entry = gtk.Entry()
+        self.g_heading_entry = Gtk.Entry()
         self.g_heading_entry.set_no_show_all(True)
-        hbox.pack_start(self.g_heading_entry)
-        self.g_heading = gtk.Label()
+        hbox.pack_start(self.g_heading_entry, True, True, 0)
+        self.g_heading = Gtk.Label()
         self.g_heading.set_alignment(0.0, 0.5)
         # FIXME escape m_name
         self.g_heading.set_markup("<b>%s</b>" % model.m_name)
-        hbox.pack_start(self.g_heading, False)
+        hbox.pack_start(self.g_heading, False, False, 0)
         #
-        button_hbox = gtk.HBox()
+        button_hbox = Gtk.HBox()
         button_hbox.set_spacing(0)
-        hbox.pack_start(button_hbox, False)
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU)
-        button = gtk.Button()
+        hbox.pack_start(button_hbox, False, False, 0)
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_EDIT, Gtk.IconSize.MENU)
+        button = Gtk.Button()
         button.add(im)
         button.connect('clicked', self.on_edit_heading)
-        button_hbox.pack_start(button, False)
+        button_hbox.pack_start(button, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
-        button = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_ADD, Gtk.IconSize.MENU)
+        button = Gtk.Button()
         button.add(im)
         button.connect('button-release-event', self.on_add)
-        button_hbox.pack_start(button, False)
+        button_hbox.pack_start(button, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
-        button = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_REMOVE, Gtk.IconSize.MENU)
+        button = Gtk.Button()
         button.add(im)
         button.connect('button-release-event', self.on_remove)
-        button_hbox.pack_start(button, False)
+        button_hbox.pack_start(button, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_CUT, gtk.ICON_SIZE_MENU)
-        b = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_CUT, Gtk.IconSize.MENU)
+        b = Gtk.Button()
         b.add(im)
         b.connect('clicked', self.on_cut)
-        button_hbox.pack_start(b, False)
+        button_hbox.pack_start(b, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_PASTE, gtk.ICON_SIZE_MENU)
-        b = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_PASTE, Gtk.IconSize.MENU)
+        b = Gtk.Button()
         b.add(im)
         b.connect('clicked', self.on_paste, -1)
         Editor.clipboard.register_paste_button(b, (pd.LinkList, pd.Page, unicode))
-        button_hbox.pack_start(b, False)
+        button_hbox.pack_start(b, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_MENU)
-        self.g_move_down_btn = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_GO_DOWN, Gtk.IconSize.MENU)
+        self.g_move_down_btn = Gtk.Button()
         self.g_move_down_btn.add(im)
         self.g_move_down_btn.connect('clicked',
             self.m_parent.move_section_down, self)
-        button_hbox.pack_start(self.g_move_down_btn, False)
+        button_hbox.pack_start(self.g_move_down_btn, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_GO_UP, gtk.ICON_SIZE_MENU)
-        self.g_move_up_btn = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_GO_UP, Gtk.IconSize.MENU)
+        self.g_move_up_btn = Gtk.Button()
         self.g_move_up_btn.add(im)
         self.g_move_up_btn.connect('clicked',
             self.m_parent.move_section_up, self)
-        button_hbox.pack_start(self.g_move_up_btn, False)
+        button_hbox.pack_start(self.g_move_up_btn, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_MENU)
-        self.g_move_left_btn = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_GO_BACK, Gtk.IconSize.MENU)
+        self.g_move_left_btn = Gtk.Button()
         self.g_move_left_btn.add(im)
         self.g_move_left_btn.connect('clicked',
             parent.m_parent.on_move_section_left, self)
-        button_hbox.pack_start(self.g_move_left_btn, False)
+        button_hbox.pack_start(self.g_move_left_btn, False, False, 0)
         #
-        im = gtk.Image()
-        im.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_MENU)
-        self.g_move_right_btn = gtk.Button()
+        im = Gtk.Image()
+        im.set_from_stock(Gtk.STOCK_GO_FORWARD, Gtk.IconSize.MENU)
+        self.g_move_right_btn = Gtk.Button()
         self.g_move_right_btn.add(im)
         self.g_move_right_btn.connect('clicked',
             parent.m_parent.on_move_section_right, self)
-        button_hbox.pack_start(self.g_move_right_btn, False)
+        button_hbox.pack_start(self.g_move_right_btn, False, False, 0)
         #
-        self.g_link_box = gtk.VBox()
-        self.pack_start(self.g_link_box, False)
+        self.g_link_box = Gtk.VBox()
+        self.pack_start(self.g_link_box, False, False, 0)
         for link in self.m_model:
-            self.g_link_box.pack_start(self.create_linkrow(link))
+            self.g_link_box.pack_start(self.create_linkrow(link), True, True, 0)
         # The button to click to add a new link
-        hbox = gtk.HBox()
-        self.pack_start(hbox)
+        hbox = Gtk.HBox()
+        self.pack_start(hbox, True, True, 0)
     def on_edit_heading(self, btn):
         self.g_heading_entry.set_text(self.m_model.m_name)
         self.g_heading_entry.show()
@@ -265,11 +263,11 @@ class Section(gtk.VBox):
             self.g_heading.show()
         sid = self.g_heading_entry.connect('activate', finish_edit)
         def keydown(entry, event):
-            if event.keyval == gtk.keysyms.Tab:
+            if event.keyval == Gdk.KEY_Tab:
                 finish_edit(entry)
         keydown_sid = self.g_heading_entry.connect('key-press-event', keydown)
         def keyup(entry, event):
-            if event.keyval == gtk.keysyms.Escape:
+            if event.keyval == Gdk.KEY_Escape:
                 self.g_heading_entry.disconnect(sid)
                 self.g_heading_entry.disconnect(keyup_id)
                 self.g_heading_entry.hide()
@@ -277,25 +275,25 @@ class Section(gtk.VBox):
                 return True
         keyup_id = self.g_heading_entry.connect('key-release-event', keyup)
     def on_add(self, btn, event):
-        menu = gtk.Menu()
-        item = gtk.MenuItem(_("Add link to new page"))
+        menu = Gtk.Menu()
+        item = Gtk.MenuItem(_("Add link to new page"))
         item.connect('activate', self.on_add_link_to_new_page)
         menu.append(item)
-        item = gtk.MenuItem(_("Add link to exercise"))
+        item = Gtk.MenuItem(_("Add link to exercise"))
         item.connect('activate', self.on_add_link)
         menu.append(item)
-        item = gtk.MenuItem(_("Add link by searching for exercises"))
+        item = Gtk.MenuItem(_("Add link by searching for exercises"))
         item.connect('activate', self.on_add_link_by_search)
         menu.append(item)
         menu.show_all()
-        menu.popup(None, None, None, event.button, event.time)
+        menu.popup(None, None, None, None, event.button, event.time)
     def on_remove(self, btn, event):
         self.m_parent.remove_section(self)
     def on_add_link_by_search(self, btn):
         dlg = SelectLessonfileBySearchDialog()
         while True:
             ret = dlg.run()
-            if ret == gtk.RESPONSE_OK:
+            if ret == Gtk.ResponseType.OK:
                 self._add_filenames([os.path.abspath(lessonfile.uri_expand(dlg.m_filename))])
             else:
                 break
@@ -309,10 +307,10 @@ class Section(gtk.VBox):
         dlg.set_current_folder(open_dir)
         while 1:
             ret = dlg.run()
-            if ret in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_CANCEL):
+            if ret in (Gtk.ResponseType.REJECT, Gtk.ResponseType.DELETE_EVENT, Gtk.ResponseType.CANCEL):
                 break
             else:
-                assert ret == gtk.RESPONSE_OK
+                assert ret == Gtk.ResponseType.OK
                 self._add_filenames(dlg.get_filenames())
                 break
         dlg.destroy()
@@ -331,13 +329,13 @@ class Section(gtk.VBox):
             except lessonfile.infocache.FileNotLessonfile:
                 continue
             self.m_model.append(fn)
-            self.g_link_box.pack_start(self.create_linkrow(fn), False)
+            self.g_link_box.pack_start(self.create_linkrow(fn, True, True, 0), False)
     def on_add_link_to_new_page(self, menuitem):
         page = pd.Page(_("Untitled%s") % "", [pd.Column()])
         self.m_model.append(page)
-        self.g_link_box.pack_start(self.create_linkrow(page))
+        self.g_link_box.pack_start(self.create_linkrow(page, True, True, 0))
     def create_linkrow(self, link_this):
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         def ff(btn, page):
             if id(page) in editor_of(self).m_page_mapping:
                 editor_of(self).show_page_id(id(page))
@@ -358,42 +356,42 @@ class Section(gtk.VBox):
                 linkbutton = gu.ClickableLabel(_(u"«%s» was not found") % link_this)
                 linkbutton.make_warning()
 
-        hbox.pack_start(linkbutton)
+        hbox.pack_start(linkbutton, True, True, 0)
         linkbutton.connect('button-press-event', self.on_right_click_row, link_this)
         hbox.show_all()
         return hbox
     def on_right_click_row(self, button, event, linked):
         idx = self.m_model.index(linked)
         if event.button == 3:
-            m = gtk.Menu()
-            item = gtk.ImageMenuItem(gtk.STOCK_DELETE)
+            m = Gtk.Menu()
+            item = Gtk.ImageMenuItem(Gtk.STOCK_DELETE)
             item.connect('activate', self.on_delete_link, linked)
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_CUT)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_CUT)
             item.connect('activate', self.on_cut_link, idx)
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_PASTE)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_PASTE)
             item.set_sensitive(bool(Editor.clipboard))
             item.connect('activate', self.on_paste, idx)
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_EDIT)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
             item.connect('activate', self.on_edit_linktext, linked)
             item.set_sensitive(bool(not isinstance(linked, basestring)))
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_GO_UP)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_GO_UP)
             item.connect('activate', self.on_move_link_up, idx)
             item.set_sensitive(bool(idx > 0))
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_GO_DOWN)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_GO_DOWN)
             item.connect('activate', self.on_move_link_down, idx)
             item.set_sensitive(bool(idx < len(self.m_model) - 1))
             m.append(item)
-            item = gtk.ImageMenuItem(gtk.STOCK_EDIT)
+            item = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
             item.set_sensitive(isinstance(linked, unicode))
             item.connect('activate', self.on_edit_file, idx)
             m.append(item)
             m.show_all()
-            m.popup(None, None, None, event.button, event.time)
+            m.popup(None, None, None, None, event.button, event.time)
             return True
     def on_delete_link(self, menuitem, linked):
         idx = self.m_model.index(linked)
@@ -406,9 +404,9 @@ class Section(gtk.VBox):
         # row is the hbox containing the linkbutton
         row = self.g_link_box.get_children()[idx]
         linkbutton = row.get_children()[0]
-        entry = gtk.Entry()
+        entry = Gtk.Entry()
         entry.set_text(linkbutton.get_label())
-        row.pack_start(entry)
+        row.pack_start(entry, True, True, 0)
         linkbutton.hide()
         entry.show()
         entry.grab_focus()
@@ -420,11 +418,11 @@ class Section(gtk.VBox):
             entry.destroy()
         sid = entry.connect('activate', finish_edit)
         def keydown(entry, event):
-            if event.keyval == gtk.keysyms.Tab:
+            if event.keyval == Gdk.KEY_Tab:
                 finish_edit(entry)
         entry.connect('key-press-event', keydown)
         def keyup(entry, event):
-            if event.keyval == gtk.keysyms.Escape:
+            if event.keyval == Gdk.KEY_Escape:
                 linkbutton.show()
                 entry.disconnect(sid)
                 entry.destroy()
@@ -454,11 +452,11 @@ class Section(gtk.VBox):
             mobj = pobj
         if idx == -1:
             self.m_model.append(mobj)
-            self.g_link_box.pack_start(self.create_linkrow(mobj))
+            self.g_link_box.pack_start(self.create_linkrow(mobj, True, True, 0))
         else:
             self.m_model.insert(idx, mobj)
             row = self.create_linkrow(mobj)
-            self.g_link_box.pack_start(row)
+            self.g_link_box.pack_start(row, True, True, 0)
             self.g_link_box.reorder_child(row, idx)
     def on_move_link_up(self, btn, idx):
         """
@@ -475,29 +473,29 @@ class Section(gtk.VBox):
         self.g_link_box.reorder_child(self.g_link_box.get_children()[idx], idx + 1)
 
 
-class Column(gtk.VBox):
+class Column(Gtk.VBox):
     def __init__(self, model, parent):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         self.set_spacing(gu.hig.SPACE_MEDIUM)
         self.m_model = model
         self.m_parent = parent
         assert isinstance(model, pd.Column)
-        self.g_section_box = gtk.VBox()
+        self.g_section_box = Gtk.VBox()
         self.g_section_box.set_spacing(gu.hig.SPACE_MEDIUM)
-        self.pack_start(self.g_section_box, False)
+        self.pack_start(self.g_section_box, False, False, 0)
         for section in model:
             assert isinstance(section, pd.LinkList)
             gui_section = Section(section, self)
-            self.g_section_box.pack_start(gui_section, False)
-        hbox = gtk.HBox()
-        self.pack_start(hbox, False)
-        b = gtk.Button(_("Add section"))
-        hbox.pack_start(b, False)
+            self.g_section_box.pack_start(gui_section, False, False, 0)
+        hbox = Gtk.HBox()
+        self.pack_start(hbox, False, False, 0)
+        b = Gtk.Button(_("Add section"))
+        hbox.pack_start(b, False, False, 0)
         b.connect('clicked', self.on_add_section)
-        b = gtk.Button(stock=gtk.STOCK_PASTE)
+        b = Gtk.Button(stock=Gtk.STOCK_PASTE)
         b.connect('clicked', self.on_paste)
         Editor.clipboard.register_paste_button(b, pd.LinkList)
-        hbox.pack_start(b, False)
+        hbox.pack_start(b, False, False, 0)
     def __del__(self):
         logging.debug("Column.__del__")
     def cut_section(self, section):
@@ -515,7 +513,7 @@ class Column(gtk.VBox):
         section = pd.LinkList(_("Untitled%s" % ""))
         self.m_model.append(section)
         gui_section = Section(section, self)
-        self.g_section_box.pack_start(gui_section, False)
+        self.g_section_box.pack_start(gui_section, False, False, 0)
         gui_section.show_all()
     def move_section_down(self, widget, section):
         idx = self.g_section_box.get_children().index(section)
@@ -541,18 +539,18 @@ class Column(gtk.VBox):
         self.m_model.append(pobj)
         sect = Section(pobj, self)
         sect.show_all()
-        self.g_section_box.pack_start(sect, False)
+        self.g_section_box.pack_start(sect, False, False, 0)
 
 
-class Page(gtk.VBox):
+class Page(Gtk.VBox):
     def __init__(self, model, parent):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         self.m_model = model
         self.m_parent = parent
-        sc = gtk.ScrolledWindow()
-        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.pack_start(sc)
-        self.g_column_box = gtk.HBox()
+        sc = Gtk.ScrolledWindow()
+        sc.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        self.pack_start(sc, True, True, 0)
+        self.g_column_box = Gtk.HBox()
         self.g_column_box.set_spacing(gu.hig.SPACE_LARGE)
         self.g_column_box.set_border_width(gu.hig.SPACE_SMALL)
         # We pack column into this box
@@ -567,7 +565,7 @@ class Page(gtk.VBox):
         self.m_model.append(column)
         gcol = Column(column, self)
         gcol.show_all()
-        self.g_column_box.pack_start(gcol)
+        self.g_column_box.pack_start(gcol, True, True, 0)
     def on_move_section_left(self, button, section):
         column_idx = self.g_column_box.get_children().index(section.m_parent)
         section_idx = section.m_parent.g_section_box.get_children().index(section)
@@ -575,7 +573,7 @@ class Page(gtk.VBox):
             to_column = self.g_column_box.get_children()[column_idx - 1]
             section.reparent(to_column.g_section_box)
             section.m_parent = to_column
-            to_column.g_section_box.set_child_packing(section, False, False, 0, gtk.PACK_START)
+            to_column.g_section_box.set_child_packing(section, False, False, 0, Gtk.PACK_START)
             self.m_model[column_idx - 1].append(self.m_model[column_idx][section_idx])
             del self.m_model[column_idx][section_idx]
             # Remove the right-most column if we moved the
@@ -594,7 +592,7 @@ class Page(gtk.VBox):
         to_column = self.g_column_box.get_children()[column_idx + 1]
         section.reparent(to_column.g_section_box)
         section.m_parent = to_column
-        to_column.g_section_box.set_child_packing(section, False, False, 0, gtk.PACK_START)
+        to_column.g_section_box.set_child_packing(section, False, False, 0, Gtk.PACK_START)
         to_section_idx = len(self.m_model[column_idx + 1])
         self.m_model[column_idx + 1].append(self.m_model[column_idx][section_idx])
         del self.m_model[column_idx][section_idx]
@@ -603,7 +601,7 @@ class Page(gtk.VBox):
         for child in self.g_column_box.get_children():
             child.destroy()
         for column in self.m_model:
-            self.g_column_box.pack_start(Column(column, self))
+            self.g_column_box.pack_start(Column(column, self), False, False, 0)
         self.g_column_box.show_all()
         self.update_buttons()
     def update_buttons(self):
@@ -639,30 +637,30 @@ class Clipboard(list):
             button.set_sensitive(bool(self) and isinstance(self[-1], types))
 
 
-class Editor(gtk.Window, gu.EditorDialogBase):
+class Editor(Gtk.Window, gu.EditorDialogBase):
     savedir = os.path.join(filesystem.user_data(), u'exercises', u'user')
     # The clipboard will be shared between all Editor instances
     clipboard = Clipboard()
     def __init__(self, filename=None):
-        gtk.Window.__init__(self)
-        logging.debug("fpeditor.Editor.__init__(%s)" % filename)
+        Gtk.Window.__init__(self)
+        logging.debug("fpeditor.Editor.__init__(%s)", filename)
         gu.EditorDialogBase.__init__(self, filename)
         self.set_default_size(800, 600)
-        self.g_main_box = gtk.VBox()
+        self.g_main_box = Gtk.VBox()
         self.add(self.g_main_box)
         self.g_actiongroup.add_actions([
-            ('GoBack', gtk.STOCK_GO_BACK, None, None, None, self.go_back),
+            ('GoBack', Gtk.STOCK_GO_BACK, None, None, None, self.go_back),
         ])
         self.setup_toolbar()
-        self.g_title_hbox = gtk.HBox()
+        self.g_title_hbox = Gtk.HBox()
         self.g_title_hbox.set_spacing(gu.hig.SPACE_SMALL)
         self.g_title_hbox.set_border_width(gu.hig.SPACE_SMALL)
-        label = gtk.Label()
+        label = Gtk.Label()
         label.set_markup(u"<b>%s</b>" % _("Front page title:"))
-        self.g_title_hbox.pack_start(label, False)
-        self.g_fptitle = gtk.Entry()
-        self.g_title_hbox.pack_start(self.g_fptitle)
-        self.g_main_box.pack_start(self.g_title_hbox, False)
+        self.g_title_hbox.pack_start(label, False, False, 0)
+        self.g_fptitle = Gtk.Entry()
+        self.g_title_hbox.pack_start(self.g_fptitle, True, True, 0)
+        self.g_main_box.pack_start(self.g_title_hbox, False, False, 0)
         # This dict maps the windows created for all pages belonging to
         # the file.
         self.m_page_mapping = {}
@@ -680,13 +678,13 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.g_fptitle.set_text(self.m_model.m_name)
         self.g_fptitle.connect('changed', self.on_frontpage_title_changed)
     def __del__(self):
-        logging.debug("fpeditor.Editor.__del__, filename=" % self.m_filename)
+        logging.debug("fpeditor.Editor.__del__, filename=%s", self.m_filename)
     def add_page(self, page):
         """
         Add and show the page.
         """
         editor_of(self).m_page_mapping[id(page.m_model)] = page
-        self.g_main_box.pack_start(page)
+        self.g_main_box.pack_start(page, True, True, 0)
         self.show_page(page)
     def show_page_id(self, page_id):
         self.show_page(self.m_page_mapping[page_id])
@@ -731,17 +729,18 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         """
         self.g_ui_manager.add_ui_from_string(uixml)
         toolbar = self.g_ui_manager.get_widget("/Toolbar")
-        self.g_main_box.pack_start(toolbar, False)
+        self.g_main_box.pack_start(toolbar, False, False, 0)
         self.g_main_box.reorder_child(toolbar, 0)
-        self.g_ui_manager.get_widget("/Toolbar").set_style(gtk.TOOLBAR_BOTH)
+        self.g_ui_manager.get_widget("/Toolbar").set_style(Gtk.ToolbarStyle.BOTH)
     def destroy_window(self, window_id):
         """
         Destroy the window with the id 'windowid' and all subwindows.
         """
         def do_del(wid):
-            for key in [k for k in self.m_page_mapping.keys()
-                    if id(parent_page(self.m_page_mapping[k]).m_model) == wid]:
-                do_del(key)
+            for key in self.m_page_mapping:
+                parent = parent_page(self.m_page_mapping[key])
+                if id(parent) == wid:
+                    do_del(key)
             editor_of(self).m_page_mapping[wid].destroy()
             del editor_of(self).m_page_mapping[wid]
         do_del(window_id)
@@ -766,8 +765,8 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         #
         if not os.path.isabs(filename):
             if not os.access(filename, os.W_OK):
-                m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
-                    gtk.BUTTONS_CLOSE, _("The front page file is write protected in your install. This is normal. If you want to edit a front page file, you have to select one of the files stored in .solfege/exercises/*/ in your home directory."))
+                m = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.INFO,
+                    Gtk.ButtonsType.CLOSE, _("The front page file is write protected in your install. This is normal. If you want to edit a front page file, you have to select one of the files stored in .solfege/exercises/*/ in your home directory."))
                 m.run()
                 m.destroy()
         self.set_not_modified()
@@ -799,19 +798,19 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         fh.save_file(self.m_filename)
         self.set_not_modified()
         # We do test for solfege.win since it is not available during testing
-        if getattr(solfege, 'win', None):
+        if hasattr(solfege, 'win'):
             solfege.win.load_frontpage()
     def on_show_help(self, *w):
         return
     def get_save_as_dialog(self):
         dialog = gu.EditorDialogBase.get_save_as_dialog(self)
-        ev2 = gtk.EventBox()
+        ev2 = Gtk.EventBox()
         ev2.set_name("DIALOGWARNING2")
-        ev = gtk.EventBox()
+        ev = Gtk.EventBox()
         ev.set_border_width(gu.hig.SPACE_SMALL)
         ev2.add(ev)
         ev.set_name("DIALOGWARNING")
-        label = gtk.Label()
+        label = Gtk.Label()
         label.set_padding(gu.hig.SPACE_MEDIUM, gu.hig.SPACE_MEDIUM)
         ev.add(label)
         label.set_markup(_("<b>IMPORTANT:</b> Your front page file <b>must</b> be saved in a subdirectory below the directory named exercises. See the user manual for details."))
@@ -820,7 +819,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         return dialog
 
 if __name__ == '__main__':
-    gtk.link_button_set_uri_hook(lambda a, b: None)
+    Gtk.link_button_set_uri_hook(lambda a, b: None)
     e = Editor()
     e.load_file("learningtrees/learningtree.txt")
-    gtk.main()
+    Gtk.main()
